@@ -34,7 +34,7 @@
 
 /* RCS version string for later identification using rcs ident {{{*/
 #ifndef lint
-static volatile const char *RCSid = "@(#) $Id: cfg.c,v 1.22 2002/11/10 20:17:56 mbr Exp $";
+static volatile const char *RCSid = "@(#) $Id: cfg.c,v 1.24 2002/12/11 09:18:44 mbr Exp $";
 #endif
 /* }}} */
 
@@ -51,29 +51,30 @@ int parse_cfg_file(char *name) { /* {{{ */
 	int argument_buffer_ptr = 0;
 	enum mode mode = MODE_INDENT;
 	config file_cfg = {
-		.daemon = -1,
-		.powermgt = -1,
+		.daemon = UNDEFINED,
+		.apm = UNDEFINED,
+		.powermgt = UNDEFINED,
 		.nvram = NULL,
-		.polltime = -1,
+		.polltime = UNDEFINED,
 		.tpbcmd = NULL,
 		.homecmd = NULL,
 		.searchcmd = NULL,
 		.mailcmd = NULL,
 		.callback = NULL,
-		.mixer = -1,
-		.mixersteps = -1,
+		.mixer = UNDEFINED,
+		.mixersteps = UNDEFINED,
 		.mixerdev = NULL,
-		.verbose = -1,
+		.verbose = UNDEFINED,
 #ifdef HAVE_LIBXOSD
-		.osd = -1,
+		.osd = UNDEFINED,
 		.osdfont = NULL,
 		.osdcolor = NULL,
-		.osdtimeout = -1,
-		.osdshadow = -1,
-		.osdvertical = -1,
-		.osdhorizontal = -1,
-		.osdpos = -1,
-		.osdalign = -1
+		.osdtimeout = UNDEFINED,
+		.osdshadow = UNDEFINED,
+		.osdvertical = UNDEFINED,
+		.osdhorizontal = UNDEFINED,
+		.osdpos = UNDEFINED,
+		.osdalign = UNDEFINED
 #endif /* HAVE_LIBXOSD */
 	};
 
@@ -177,7 +178,8 @@ int parse_cfg_file(char *name) { /* {{{ */
 void init_cfg(void) { /* {{{ */
 
 	/* set all values to defaults */
-	cfg.powermgt = STATE_AUTO;
+	cfg.apm = DEFAULT_APM;
+	cfg.powermgt = DEFAULT_POWERMGT;
 	if((cfg.nvram = strdup(DEFAULT_NVRAMDEV)) == NULL) {
 		fprintf(stderr, "Not enough memory");
 		_exit(1);
@@ -188,15 +190,15 @@ void init_cfg(void) { /* {{{ */
 	cfg.searchcmd = NULL;
 	cfg.mailcmd = NULL;
 	cfg.callback = NULL;
-	cfg.mixer = STATE_OFF;
+	cfg.mixer = DEFAULT_MIXER;
 	cfg.mixersteps = DEFAULT_MIXERSTEPS;
 	if((cfg.mixerdev = strdup(DEFAULT_MIXERDEV)) == NULL) {
 		fprintf(stderr, "Not enough memory");
 		_exit(1);
 	}
-	cfg.verbose = STATE_OFF;
+	cfg.verbose = DEFAULT_VERBOSE;
 #ifdef HAVE_LIBXOSD
-	cfg.osd = STATE_ON;
+	cfg.osd = DEFAULT_OSD;
 	if((cfg.osdfont = strdup(DEFAULT_OSDFONT)) == NULL) {
 		fprintf(stderr, "Not enough memory");
 		_exit(1);
@@ -218,11 +220,15 @@ void init_cfg(void) { /* {{{ */
 
 void override_cfg(config *master) { /* {{{ */
 	/* override the defaults if the are set in master struct */
-	if(master->daemon != -1) {
+	if(master->daemon != UNDEFINED) {
 		cfg.daemon = master->daemon;
 	}
 
-	if(master->powermgt != -1) {
+	if(master->apm != UNDEFINED) {
+		cfg.apm = master->apm;
+	}
+
+	if(master->powermgt != UNDEFINED) {
 		cfg.powermgt = master->powermgt;
 	}
 
@@ -233,7 +239,7 @@ void override_cfg(config *master) { /* {{{ */
 		cfg.nvram = master->nvram;
 	}
 
-	if(master->polltime != -1) {
+	if(master->polltime != UNDEFINED) {
 		cfg.polltime = master->polltime;
 	}
 
@@ -272,11 +278,11 @@ void override_cfg(config *master) { /* {{{ */
 		cfg.callback = master->callback;
 	}
 
-	if(master->mixer != -1) {
+	if(master->mixer != UNDEFINED) {
 		cfg.mixer = master->mixer;
 	}
 
-	if(master->mixersteps != -1) {
+	if(master->mixersteps != UNDEFINED) {
 		cfg.mixersteps = master->mixersteps;
 	}
 
@@ -287,12 +293,12 @@ void override_cfg(config *master) { /* {{{ */
 		cfg.mixerdev = master->mixerdev;
 	}
 
-	if(master->verbose != -1) {
+	if(master->verbose != UNDEFINED) {
 		cfg.verbose = master->verbose;
 	}
 
 #ifdef HAVE_LIBXOSD
-	if(master->osd != -1) {
+	if(master->osd != UNDEFINED) {
 		cfg.osd = master->osd;
 	}
 
@@ -310,27 +316,27 @@ void override_cfg(config *master) { /* {{{ */
 		cfg.osdcolor = master->osdcolor;
 	}
 
-	if(master->osdtimeout != -1) {
+	if(master->osdtimeout != UNDEFINED) {
 		cfg.osdtimeout = master->osdtimeout;
 	}
 
-	if(master->osdshadow != -1) {
+	if(master->osdshadow != UNDEFINED) {
 		cfg.osdshadow = master->osdshadow;
 	}
 
-	if(master->osdvertical != -1) {
+	if(master->osdvertical != UNDEFINED) {
 		cfg.osdvertical = master->osdvertical;
 	}
 
-	if(master->osdhorizontal != -1) {
+	if(master->osdhorizontal != UNDEFINED) {
 		cfg.osdhorizontal = master->osdhorizontal;
 	}
 
-	if(master->osdpos != -1) {
+	if(master->osdpos != UNDEFINED) {
 		cfg.osdpos = master->osdpos;
 	}
 
-	if(master->osdalign != -1) {
+	if(master->osdalign != UNDEFINED) {
 		cfg.osdalign = master->osdalign;
 	}
 #endif /* HAVE_LIBXOSD */
@@ -341,7 +347,27 @@ void override_cfg(config *master) { /* {{{ */
 void set_value(char *key, char *arg, config *cfg) { /* {{{ */
 	char *endptr;
 
-	if (strcmp(CFG_POWERMGT, key) == 0) { /* {{{ */
+	if (strcmp(CFG_APM, key) == 0) { /* {{{ */
+		int i = 0;
+		while(arg[i]) {
+			arg[i] = tolower(arg[i]);
+			i++;
+		}
+		if(strcmp(CFG_APM_ON, arg) == 0) {
+			cfg->apm = STATE_ON;
+		}
+		else {
+			if(strcmp(CFG_APM_OFF, arg) == 0) {
+				cfg->apm = STATE_OFF;
+			}
+			else {
+				fprintf(stderr, "Illegal apm state %s\n", arg);
+				_exit(1);
+			}
+		}
+	} /* }}} */
+
+	else if (strcmp(CFG_POWERMGT, key) == 0) { /* {{{ */
 		int i=0;
 		while(arg[i]) {
 			arg[i] = tolower(arg[i]);
@@ -617,27 +643,28 @@ void set_value(char *key, char *arg, config *cfg) { /* {{{ */
 
 void parse_option(int argc, char **argv) { /* {{{ */
 	config cmd_cfg = {
-		.daemon = -1,
-		.powermgt = -1,
+		.daemon = UNDEFINED,
+		.apm = UNDEFINED,
+		.powermgt = UNDEFINED,
 		.nvram = NULL,
-		.polltime = -1,
+		.polltime = UNDEFINED,
 		.tpbcmd = NULL,
 		.homecmd = NULL,
 		.searchcmd = NULL,
 		.mailcmd = NULL,
 		.callback = NULL,
-		.mixer = -1,
-		.mixersteps = -1,
+		.mixer = UNDEFINED,
+		.mixersteps = UNDEFINED,
 		.mixerdev = NULL,
-		.verbose = -1,
+		.verbose = UNDEFINED,
 #ifdef HAVE_LIBXOSD
-		.osd = -1,
+		.osd = UNDEFINED,
 		.osdfont = NULL,
 		.osdcolor = NULL,
-		.osdtimeout = -1,
-		.osdvertical = -1,
-		.osdpos = -1,
-		.osdalign = -1
+		.osdtimeout = UNDEFINED,
+		.osdvertical = UNDEFINED,
+		.osdpos = UNDEFINED,
+		.osdalign = UNDEFINED
 #endif /* HAVE_LIBXOSD */
 	};
 
@@ -647,6 +674,7 @@ void parse_option(int argc, char **argv) { /* {{{ */
 		static struct option long_options[] = {
 			{"help",     0, NULL, 'h'},
 			{"daemon",   0, NULL, 'd'},
+			{"apm",      1, NULL, 'A'},
 			{"powermgt", 1, NULL, 'P'},
 			{"config",   1, NULL, 'c'},
 			{"callback", 1, NULL, 'C'},
@@ -664,9 +692,9 @@ void parse_option(int argc, char **argv) { /* {{{ */
 		};
 
 #ifdef HAVE_LIBXOSD
-		next_option = getopt_long (argc, argv, "hdP:c:C:m:o:p:t:H:S:M:v", long_options, &option_index);
+		next_option = getopt_long (argc, argv, "hdA:P:c:C:m:o:p:t:H:S:M:v", long_options, &option_index);
 #else
-		next_option = getopt_long (argc, argv, "hdP:c:C:m:p:t:H:S:M:v", long_options, &option_index);
+		next_option = getopt_long (argc, argv, "hdA:P:c:C:m:p:t:H:S:M:v", long_options, &option_index);
 #endif /* HAVE_LIBXOSD */
 
 		if (next_option == -1) {
@@ -683,6 +711,10 @@ void parse_option(int argc, char **argv) { /* {{{ */
 				cmd_cfg.daemon = STATE_ON;
 				break;
 
+			case 'A':
+				set_value(CFG_APM, optarg, &cmd_cfg);
+				break;
+
 			case 'P':
 				set_value(CFG_POWERMGT, optarg, &cmd_cfg);
 				break;
@@ -765,6 +797,7 @@ void print_usage(const char *prog) /* {{{ */
 	printf("   -C, --callback=CMD   string with command that should be executed for each\n");
 	printf("                        pressed button. It is called with pressed button as\n");
 	printf("                        first argument and new state as second [none]\n");
+	printf("   -A, --apm=STATE      poll /proc/apm for apm information [on]\n");
 	printf("   -P, --powermgt=STATE display of power management messages [auto]\n");
 	printf("   -v, --verbose        print information about pressed keys\n");
 	return;
