@@ -21,17 +21,22 @@
 #ifndef __CFG_H__
 #define __CFG_H__
 
-enum state {STATE_OFF, STATE_ON, STATE_AUTO};
+enum state {UNDEFINED = -1, STATE_OFF, STATE_ON, STATE_AUTO};
 enum mode {MODE_COMMENT, MODE_INDENT, MODE_KEYWORD, MODE_SEPARATOR, MODE_ARGUMENT, MODE_END};
 
 /* configuration defaults */
+#define DEFAULT_APM STATE_OFF
+#define DEFAULT_POWERMGT STATE_AUTO
+#define DEFAULT_VERBOSE STATE_OFF
 #define DEFAULT_POLLTIME 1000
 #define DEFAULT_NVRAMDEV "/dev/nvram"
+#define DEFAULT_MIXER STATE_OFF
 #define DEFAULT_MIXERSTEPS 20
 #define DEFAULT_MIXERDEV "/dev/mixer"
 #ifdef HAVE_LIBXOSD
+#define DEFAULT_OSD STATE_ON
 #define DEFAULT_OSDLINES 2
-#define DEFAULT_OSDFONT "-*-lucidatypewriter-medium-r-normal-*-*-250-*-*-*-*-*-*"
+#define DEFAULT_OSDFONT "10x20"
 #define DEFAULT_OSDCOLOR "BLUE"
 #define DEFAULT_OSDTIMEOUT 3
 #define DEFAULT_OSDSHADOW 2
@@ -50,6 +55,9 @@ enum mode {MODE_COMMENT, MODE_INDENT, MODE_KEYWORD, MODE_SEPARATOR, MODE_ARGUMEN
 #define PRIVATE_CONFIG_FILE ".tpbrc"
 
 /* Keywords in configuration file */
+#define CFG_APM "apm"
+#define CFG_APM_ON "on"
+#define CFG_APM_OFF "off"
 #define CFG_POWERMGT "powermgt"
 #define CFG_POWERMGT_ON "on"
 #define CFG_POWERMGT_OFF "off"
@@ -91,6 +99,7 @@ enum mode {MODE_COMMENT, MODE_INDENT, MODE_KEYWORD, MODE_SEPARATOR, MODE_ARGUMEN
 
 typedef struct {
 	int daemon;
+	int apm;
 	int powermgt;
 	char *nvram;
 	int polltime;
