@@ -39,9 +39,9 @@
 #include "tpb.h"
 /* }}} */
 
-/* RCS version string for later identification using rcs ident {{{*/
+/* RCS version string for later identification using rcs ident {{{ */
 #ifndef lint
-static volatile const char *RCSid = "@(#) $Id: tpb.c,v 1.33 2002/11/18 14:43:41 mbr Exp $";
+static volatile const char *RCSid = "@(#) $Id: tpb.c,v 1.38 2002/12/11 13:15:44 mbr Exp $";
 #endif
 /* }}} */
 
@@ -50,6 +50,10 @@ extern config cfg;
 /* }}} */
 
 /* function prototypes {{{ */
+void daemonize(void);
+#ifdef HAVE_LIBXOSD
+xosd *init_xosd(void);
+#endif /* HAVE_LIBXOSD */
 int get_state(t_nvram_state *nvram_state);
 int get_apm_state(t_nvram_state *nvram_state);
 int fork_app(char * cmd);
@@ -58,10 +62,9 @@ int change_volume(int change);
 int apmiser_running(void);
 /* }}} */
 
-int main(int argc, char **argv)/* {{{ */
+int main(int argc, char **argv) /* {{{ */
 {
 	t_nvram_state nvram_state, last_nvram_state;
-	int filedes;
 	int vol = 0;
 	int display_state = 1;
 	char *home;
@@ -102,135 +105,17 @@ int main(int argc, char **argv)/* {{{ */
 	/* parse options */
 	parse_option(argc, argv);
 
-	/* become a daemon if requested {{{ */
+	/* become a daemon if requested */
 	if(cfg.daemon == STATE_ON) {
-		/* code inspired by comp.unix.programmer FAQ */
-		switch (fork())
-		{
-			case 0:
-				break;
-			case -1:
-				fprintf(stderr, "Unable to fork daemon");
-				perror(NULL);
-				return -1;
-			default:
-				_exit(0);          /* exit the original process */
-		}
+		daemonize();
+	}
 
-		/* create a new session */
-		if (setsid() < 0) {
-			return -1;
-		}
-
-		switch (fork())
-		{
-			case 0:
-				break;
-			case -1:
-				fprintf(stderr, "Unable to fork daemon");
-				perror(NULL);
-				return -1;
-			default:
-				_exit(0);
-		}
-
-		if(cfg.verbose == STATE_ON) {
-			puts("Daemon started.");
-		}
-
-
-		/* change to root directory */
-		chdir("/");
-
-		/* close all filedescriptors */
-		for(filedes = 0 ; filedes < sysconf(_SC_OPEN_MAX) ; filedes++) {
-			close(filedes);
-		}
-		
-		/* open /dev/null as file descriptor 0 */
-		open("/dev/null",O_RDWR);
-
-		/* duplicate file descriptor 0 to file descriptor 1 (next free) */
-		dup(0);
-
-		/* duplicate file descriptor 0 to file descriptor 2 (next free) */
-		dup(0);
-	} /* }}} */
-
-	/* initialize osd {{{ */
+	/* initialize osd */
 #ifdef HAVE_LIBXOSD
 	if(cfg.osd == STATE_ON) {
-#ifdef HAVE_LIBXOSD0
-		osd_ptr = xosd_init(DEFAULT_OSDFONT, DEFAULT_OSDCOLOR, DEFAULT_OSDTIMEOUT,
-												DEFAULT_OSDPOS, DEFAULT_OSDVERTICAL, DEFAULT_OSDSHADOW);
-#elif HAVE_LIBXOSD1
-		osd_ptr = xosd_init(DEFAULT_OSDFONT, DEFAULT_OSDCOLOR, DEFAULT_OSDTIMEOUT,
-												DEFAULT_OSDPOS, DEFAULT_OSDVERTICAL, DEFAULT_OSDSHADOW, DEFAULT_OSDLINES);
-#elif HAVE_LIBXOSD2
-		osd_ptr = xosd_create(DEFAULT_OSDLINES);
-#endif
-		if(osd_ptr == NULL) {
-			fprintf(stderr, "Unable to initialize xosd. Running without onsceen display.\n");
-		}
-		else {
-			if(xosd_set_font(osd_ptr, cfg.osdfont)) {
-				fprintf(stderr, "Invalid font \"%s\" for xosd. Running with the default \"%s\".\n",
-								cfg.osdfont, DEFAULT_OSDFONT);
-				xosd_set_font(osd_ptr, DEFAULT_OSDFONT);
-			}
-			
-			if(xosd_set_colour(osd_ptr, cfg.osdcolor)) {
-				fprintf(stderr, "Invalid color \"%s\" for xosd. Running with the default \"%s\".\n",
-								cfg.osdcolor, DEFAULT_OSDCOLOR);
-				xosd_set_colour(osd_ptr, DEFAULT_OSDCOLOR);
-			}
-
-			if(xosd_set_timeout(osd_ptr, cfg.osdtimeout)) {
-				fprintf(stderr, "Invalid timeout \"%d\" for xosd. Running with the default \"%d\".\n",
-								cfg.osdtimeout, DEFAULT_OSDTIMEOUT);
-				xosd_set_timeout(osd_ptr, DEFAULT_OSDTIMEOUT);
-			}
-
-			if(xosd_set_pos(osd_ptr, cfg.osdpos)) {
-				fprintf(stderr, "Invalid position \"%d\" for xosd. Running with the default \"%d\".\n",
-								cfg.osdpos, DEFAULT_OSDPOS);
-				xosd_set_pos(osd_ptr, DEFAULT_OSDPOS);
-			}
-
-#ifdef HAVE_LIBXOSD2
-			if(xosd_set_horizontal_offset(osd_ptr, cfg.osdhorizontal)) {
-				fprintf(stderr, "Invalid horizontal offset \"%d\" for xosd. Running with the default \"%d\".\n",
-								cfg.osdhorizontal, DEFAULT_OSDHORIZONTAL);
-				xosd_set_horizontal_offset(osd_ptr, DEFAULT_OSDHORIZONTAL);
-			}
-
-			if(xosd_set_vertical_offset(osd_ptr, cfg.osdvertical)) {
-				fprintf(stderr, "Invalid vertical offset \"%d\" for xosd. Running with the default \"%d\".\n",
-								cfg.osdvertical, DEFAULT_OSDVERTICAL);
-				xosd_set_vertical_offset(osd_ptr, DEFAULT_OSDVERTICAL);
-			}
-#else
-			if(xosd_set_offset(osd_ptr, cfg.osdvertical)) {
-				fprintf(stderr, "Invalid vertical offset \"%d\" for xosd. Running with the default \"%d\".\n",
-								cfg.osdvertical, DEFAULT_OSDVERTICAL);
-				xosd_set_offset(osd_ptr, DEFAULT_OSDVERTICAL);
-			}
-#endif
-
-			if(xosd_set_shadow_offset(osd_ptr, cfg.osdshadow)) {
-				fprintf(stderr, "Invalid shadow offset \"%d\" for xosd. Running with the default \"%d\".\n",
-								cfg.osdshadow, DEFAULT_OSDSHADOW);
-				xosd_set_shadow_offset(osd_ptr, DEFAULT_OSDSHADOW);
-			}
-
-			if(xosd_set_align(osd_ptr, cfg.osdalign)) {
-				fprintf(stderr, "Invalid alignment \"%d\" for xosd. Running with default \"%d\".\n",
-								cfg.osdalign, DEFAULT_OSDALIGN);
-				xosd_set_align(osd_ptr, DEFAULT_OSDALIGN);
-			}
-		}
+		osd_ptr = init_xosd();
 	}
-#endif /* HAVE_LIBXOSD */ /* }}} */
+#endif /* HAVE_LIBXOSD */
 
 	/* to initialize struct */
 	if(get_state(&nvram_state)) {
@@ -627,9 +512,178 @@ int main(int argc, char **argv)/* {{{ */
 	}
 
 	_exit(0); /* never reached */
-}/*}}}*/
+} /* }}} */
 
-int get_state(t_nvram_state *nvram_state)/*{{{*/
+void daemonize(void) /* {{{ */
+{
+	int filedes;
+
+	/* code inspired by comp.unix.programmer FAQ */
+	switch (fork())
+	{
+		case 0:
+			break;
+		case -1:
+			fprintf(stderr, "Unable to fork daemon");
+			perror(NULL);
+			_exit(-1);
+		default:
+			_exit(0);          /* exit the original process */
+	}
+
+	/* create a new session */
+	if (setsid() < 0) {
+		_exit(-1);
+	}
+
+	switch (fork())
+	{
+		case 0:
+			break;
+		case -1:
+			fprintf(stderr, "Unable to fork daemon");
+			perror(NULL);
+			_exit(-1);
+		default:
+			_exit(0);
+	}
+
+	if(cfg.verbose == STATE_ON) {
+		puts("Daemon started.");
+	}
+
+	/* change to root directory */
+	chdir("/");
+
+	/* close all filedescriptors */
+	for(filedes = 0 ; filedes < sysconf(_SC_OPEN_MAX) ; filedes++) {
+		close(filedes);
+	}
+
+	/* open /dev/null as file descriptor 0 */
+	open("/dev/null",O_RDWR);
+
+	/* duplicate file descriptor 0 to file descriptor 1 (next free) */
+	dup(0);
+
+	/* duplicate file descriptor 0 to file descriptor 2 (next free) */
+	dup(0);
+
+	return;
+} /* }}} */
+
+#ifdef HAVE_LIBXOSD
+xosd *init_xosd(void) /* {{{ */
+{
+	xosd *osd_ptr = NULL;
+
+#ifdef HAVE_LIBXOSD0
+	osd_ptr = xosd_init(DEFAULT_OSDFONT, DEFAULT_OSDCOLOR, DEFAULT_OSDTIMEOUT,
+											DEFAULT_OSDPOS, DEFAULT_OSDVERTICAL, DEFAULT_OSDSHADOW);
+#elif HAVE_LIBXOSD1
+	osd_ptr = xosd_init(DEFAULT_OSDFONT, DEFAULT_OSDCOLOR, DEFAULT_OSDTIMEOUT,
+											DEFAULT_OSDPOS, DEFAULT_OSDVERTICAL, DEFAULT_OSDSHADOW, DEFAULT_OSDLINES);
+#elif HAVE_LIBXOSD2
+	osd_ptr = xosd_create(DEFAULT_OSDLINES);
+#endif
+
+	if(osd_ptr == NULL) {
+		fprintf(stderr, "Unable to initialize xosd. Running without onsceen display.\n");
+	}
+	else {
+		/* initialize font */
+		if(xosd_set_font(osd_ptr, cfg.osdfont)) {
+			if(xosd_set_font(osd_ptr, DEFAULT_OSDFONT)) {
+				/* even the default font is not available */
+				fprintf(stderr, "Invalid xosd font \"%s\". Even the default font \"%s\" ",
+								cfg.osdfont, DEFAULT_OSDFONT);
+				fprintf(stderr, "is not available. Please add an existing font to your %s file.\n",
+								GLOBAL_CONFIG_FILE);
+				fprintf(stderr, "Running without onsceen display.\n");
+
+#if defined(HAVE_LIBXOSD0) || defined(HAVE_LIBXOSD1)
+				/* Disabled because of a bug in xosd lib
+				 * xosd_uninit(osd_ptr);
+				 */
+#elif HAVE_LIBXOSD2
+				/* Disabled because of a bug in xosd lib
+				 * xosd_destroy(osd_ptr);
+				 */
+#endif
+
+				return NULL;
+			}
+			else {
+				/* we run with the default font */
+				fprintf(stderr, "Invalid xosd font \"%s\". Running with the default \"%s\".\n",
+								cfg.osdfont, DEFAULT_OSDFONT);
+			}
+		}
+
+		/* initialize color */
+		if(xosd_set_colour(osd_ptr, cfg.osdcolor)) {
+			fprintf(stderr, "Invalid xosd color \"%s\". Running with the default \"%s\".\n",
+							cfg.osdcolor, DEFAULT_OSDCOLOR);
+			xosd_set_colour(osd_ptr, DEFAULT_OSDCOLOR);
+		}
+
+		/* initialize timeout */
+		if(xosd_set_timeout(osd_ptr, cfg.osdtimeout)) {
+			fprintf(stderr, "Invalid xosd timeout \"%d\". Running with the default \"%d\".\n",
+							cfg.osdtimeout, DEFAULT_OSDTIMEOUT);
+			xosd_set_timeout(osd_ptr, DEFAULT_OSDTIMEOUT);
+		}
+
+		/* initialize position */
+		if(xosd_set_pos(osd_ptr, cfg.osdpos)) {
+			fprintf(stderr, "Invalid xosd position \"%d\". Running with the default \"%d\".\n",
+							cfg.osdpos, DEFAULT_OSDPOS);
+			xosd_set_pos(osd_ptr, DEFAULT_OSDPOS);
+		}
+
+#ifdef HAVE_LIBXOSD2
+		/* initialize horizontal offset */
+		if(xosd_set_horizontal_offset(osd_ptr, cfg.osdhorizontal)) {
+			fprintf(stderr, "Invalid horizontal xosd offset \"%d\". Running with the default \"%d\".\n",
+							cfg.osdhorizontal, DEFAULT_OSDHORIZONTAL);
+			xosd_set_horizontal_offset(osd_ptr, DEFAULT_OSDHORIZONTAL);
+		}
+
+		/* initialize vertical offset */
+		if(xosd_set_vertical_offset(osd_ptr, cfg.osdvertical)) {
+			fprintf(stderr, "Invalid vertical xosd offset \"%d\". Running with the default \"%d\".\n",
+							cfg.osdvertical, DEFAULT_OSDVERTICAL);
+			xosd_set_vertical_offset(osd_ptr, DEFAULT_OSDVERTICAL);
+		}
+#else
+		/* initialize vertical offset */
+		if(xosd_set_offset(osd_ptr, cfg.osdvertical)) {
+			fprintf(stderr, "Invalid vertical xosd offset \"%d\". Running with the default \"%d\".\n",
+							cfg.osdvertical, DEFAULT_OSDVERTICAL);
+			xosd_set_offset(osd_ptr, DEFAULT_OSDVERTICAL);
+		}
+#endif
+
+		/* initialize shadow offset */
+		if(xosd_set_shadow_offset(osd_ptr, cfg.osdshadow)) {
+			fprintf(stderr, "Invalid shadow xosd offset \"%d\". Running with the default \"%d\".\n",
+							cfg.osdshadow, DEFAULT_OSDSHADOW);
+			xosd_set_shadow_offset(osd_ptr, DEFAULT_OSDSHADOW);
+		}
+
+		/* initialize alignment */
+		if(xosd_set_align(osd_ptr, cfg.osdalign)) {
+			fprintf(stderr, "Invalid xosd alignment \"%d\". Running with default \"%d\".\n",
+							cfg.osdalign, DEFAULT_OSDALIGN);
+			xosd_set_align(osd_ptr, DEFAULT_OSDALIGN);
+		}
+	}
+
+	return osd_ptr;
+} /* }}} */
+#endif /* HAVE_LIBXOSD */
+
+int get_state(t_nvram_state *nvram_state) /* {{{ */
 {
 	int  fdsc;
 	char buffer[114];
@@ -674,10 +728,12 @@ int get_state(t_nvram_state *nvram_state)/*{{{*/
 	nvram_state->powermgt_ac       = ( buffer[0x39] & 0x07);
 	nvram_state->powermgt_battery  = ( buffer[0x39] & 0x38) >> 3;
 
-	get_apm_state(nvram_state);
+	if(cfg.apm == STATE_ON) {
+		get_apm_state(nvram_state);
+	}
 
 	return 0;
-}/*}}}*/
+} /* }}} */
 
 int get_apm_state(t_nvram_state *nvram_state) {  /* {{{ */
 	int i;
